@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016  David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "bookmarksicon.h"
 #include "bookmarks.h"
 #include "bookmarkitem.h"
+#include "bookmarkstoolbar.h"
 #include "siteicon.h"
 #include "goicon.h"
 #include "downicon.h"
@@ -32,7 +33,6 @@
 #include "qzsettings.h"
 #include "colors.h"
 #include "autofillicon.h"
-#include "searchenginesmanager.h"
 #include "completer/locationcompleter.h"
 
 #include <QTimer>
@@ -73,7 +73,7 @@ LocationBar::LocationBar(BrowserWindow* window)
     m_completer = new LocationCompleter(this);
     m_completer->setMainWindow(m_window);
     m_completer->setLocationBar(this);
-    connect(m_completer, SIGNAL(showCompletion(QString)), this, SLOT(showCompletion(QString)));
+    connect(m_completer, SIGNAL(showCompletion(QString,bool)), this, SLOT(showCompletion(QString,bool)));
     connect(m_completer, SIGNAL(showDomainCompletion(QString)), this, SLOT(showDomainCompletion(QString)));
     connect(m_completer, SIGNAL(loadCompletion()), this, SLOT(requestLoadUrl()));
     connect(m_completer, SIGNAL(clearCompletion()), this, SLOT(clearCompletion()));
@@ -129,7 +129,7 @@ void LocationBar::setWebView(TabbedWebView* view)
     connect(m_webView, SIGNAL(loadFinished(bool)), SLOT(loadFinished()));
     connect(m_webView, SIGNAL(urlChanged(QUrl)), this, SLOT(showUrl(QUrl)));
     connect(m_webView, SIGNAL(privacyChanged(bool)), this, SLOT(setPrivacyState(bool)));
-    connect(m_webView, SIGNAL(iconChanged()), this, SLOT(updateSiteIcon()));
+    connect(m_webView, &TabbedWebView::iconChanged, this, &LocationBar::updateSiteIcon);
 }
 
 void LocationBar::setText(const QString &text)
@@ -145,20 +145,21 @@ void LocationBar::setText(const QString &text)
 void LocationBar::updatePlaceHolderText()
 {
     if (qzSettings->searchFromAddressBar) {
-        QString engineName = qzSettings->searchWithDefaultEngine ?
-                             mApp->searchEnginesManager()->defaultEngine().name :
-                             mApp->searchEnginesManager()->activeEngine().name;
-        setPlaceholderText(tr("Enter URL address or search on %1").arg(engineName));
+        setPlaceholderText(tr("Enter address or search with %1").arg(searchEngine().name));
     } else
-        setPlaceholderText(tr("Enter URL address"));
+        setPlaceholderText(tr("Enter address"));
 }
 
-void LocationBar::showCompletion(const QString &completion)
+void LocationBar::showCompletion(const QString &completion, bool completeDomain)
 {
     LineEdit::setText(completion);
 
     // Move cursor to the end
     end(false);
+
+    if (completeDomain) {
+        completer()->complete();
+    }
 }
 
 void LocationBar::clearCompletion()
@@ -208,8 +209,7 @@ LoadRequest LocationBar::createLoadRequest() const
         // would convert it to QUrl("http://WORD")
         if (!t.contains(QL1C(' ')) && !t.contains(QL1C('.'))) {
             req.setUrl(QUrl(t));
-        }
-        else {
+        } else {
             const QUrl &guessed = QUrl::fromUserInput(t);
             if (!guessed.isEmpty())
                 req.setUrl(guessed);
@@ -218,6 +218,11 @@ LoadRequest LocationBar::createLoadRequest() const
         }
     }
 
+    // Search when creating url failed
+    if (!req.url().isValid()) {
+        req = mApp->searchEnginesManager()->searchResult(t);
+    }
+
     return req;
 }
 
@@ -230,13 +235,24 @@ QString LocationBar::convertUrlToText(const QUrl &url)
 
     QString stringUrl = QzTools::urlEncodeQueryString(url);
 
-    if (stringUrl == QL1S("qupzilla:speeddial") || stringUrl.startsWith(QL1S("qupzilla:adblock")) || stringUrl == QL1S("about:blank")) {
+    if (stringUrl == QL1S("qupzilla:speeddial") || stringUrl == QL1S("about:blank")) {
         stringUrl.clear();
     }
 
     return stringUrl;
 }
 
+SearchEnginesManager::Engine LocationBar::searchEngine()
+{
+    if (!qzSettings->searchFromAddressBar) {
+        return SearchEnginesManager::Engine();
+    } else if (qzSettings->searchWithDefaultEngine) {
+        return mApp->searchEnginesManager()->defaultEngine();
+    } else {
+        return mApp->searchEnginesManager()->activeEngine();
+    }
+}
+
 void LocationBar::refreshTextFormat()
 {
     if (!m_webView) {
@@ -403,6 +419,10 @@ void LocationBar::focusInEvent(QFocusEvent* event)
 
     clearTextFormat();
     LineEdit::focusInEvent(event);
+
+    if (Settings().value("Browser-View-Settings/instantBookmarksToolbar").toBool()) {
+        m_window->bookmarksToolbar()->show();
+    }
 }
 
 void LocationBar::focusOutEvent(QFocusEvent* event)
@@ -422,12 +442,16 @@ void LocationBar::focusOutEvent(QFocusEvent* event)
     }
 
     refreshTextFormat();
+
+    if (Settings().value("Browser-View-Settings/instantBookmarksToolbar").toBool()) {
+        m_window->bookmarksToolbar()->hide();
+    }
 }
 
 void LocationBar::dropEvent(QDropEvent* event)
 {
     if (event->mimeData()->hasUrls()) {
-        QUrl dropUrl = event->mimeData()->urls().at(0);
+        const QUrl dropUrl = event->mimeData()->urls().at(0);
         if (WebView::isUrlValid(dropUrl)) {
             setText(dropUrl.toString());
 
@@ -440,7 +464,8 @@ void LocationBar::dropEvent(QDropEvent* event)
         }
     }
     else if (event->mimeData()->hasText()) {
-        QUrl dropUrl = QUrl(event->mimeData()->text().trimmed());
+        const QString dropText = event->mimeData()->text().trimmed();
+        const QUrl dropUrl = QUrl(dropText);
         if (WebView::isUrlValid(dropUrl)) {
             setText(dropUrl.toString());
 
@@ -450,8 +475,11 @@ void LocationBar::dropEvent(QDropEvent* event)
             QFocusEvent event(QFocusEvent::FocusOut);
             LineEdit::focusOutEvent(&event);
             return;
+        } else {
+            setText(dropText);
+            setFocus();
+            return;
         }
-
     }
 
     LineEdit::dropEvent(event);
