@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016 David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -25,11 +25,11 @@
 #include "mainapplication.h"
 #include "pluginproxy.h"
 #include "iconprovider.h"
+#include "tabcontextmenu.h"
 
 #include <QMenu>
 #include <QMimeData>
 #include <QMouseEvent>
-#include <QMessageBox>
 #include <QStyleOption>
 #include <QApplication>
 #include <QTimer>
@@ -44,7 +44,6 @@ TabBar::TabBar(BrowserWindow* window, TabWidget* tabWidget)
     , m_tabWidget(tabWidget)
     , m_hideTabBarWithOneTab(false)
     , m_showCloseOnInactive(0)
-    , m_clickedTab(0)
     , m_normalTabWidth(0)
     , m_activeTabWidth(0)
     , m_forceHidden(false)
@@ -127,16 +126,6 @@ void TabBar::overflowChanged(bool overflowed)
     }
 }
 
-void TabBar::closeAllButCurrent()
-{
-    QMessageBox::StandardButton button = QMessageBox::question(this, tr("Close Tabs"), tr("Do you really want to close other tabs?"),
-                                         QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
-
-    if (button == QMessageBox::Yes) {
-        emit closeAllButCurrent(m_clickedTab);
-    }
-}
-
 QSize TabBar::tabSizeHint(int index, bool fast) const
 {
     if (!m_window->isVisible()) {
@@ -253,9 +242,6 @@ QSize TabBar::tabSizeHint(int index, bool fast) const
 
 int TabBar::comboTabBarPixelMetric(ComboTabBar::SizeType sizeType) const
 {
-    if (!isVisible())
-        return -1;
-
     switch (sizeType) {
     case ComboTabBar::PinnedTabWidth:
         return iconButtonSize().width() + style()->pixelMetric(QStyle::PM_TabBarTabHSpace, 0, this);
@@ -305,50 +291,8 @@ void TabBar::showCloseButton(int index)
 void TabBar::contextMenuEvent(QContextMenuEvent* event)
 {
     int index = tabAt(event->pos());
-    m_clickedTab = index;
 
-    QMenu menu;
-    menu.addAction(IconProvider::newTabIcon(), tr("&New tab"), m_window, SLOT(addTab()));
-    menu.addSeparator();
-    if (index != -1) {
-        WebTab* webTab = qobject_cast<WebTab*>(m_tabWidget->widget(m_clickedTab));
-        if (!webTab) {
-            return;
-        }
-
-        if (m_window->weView(m_clickedTab)->isLoading()) {
-            menu.addAction(QIcon::fromTheme(QSL("process-stop")), tr("&Stop Tab"), this, SLOT(stopTab()));
-        }
-        else {
-            menu.addAction(QIcon::fromTheme(QSL("view-refresh")), tr("&Reload Tab"), this, SLOT(reloadTab()));
-        }
-
-        menu.addAction(QIcon::fromTheme("tab-duplicate"), tr("&Duplicate Tab"), this, SLOT(duplicateTab()));
-
-        if (count() > 1 && !webTab->isPinned()) {
-            menu.addAction(QIcon::fromTheme("tab-detach"), tr("D&etach Tab"), this, SLOT(detachTab()));
-        }
-
-        menu.addAction(webTab->isPinned() ? tr("Un&pin Tab") : tr("&Pin Tab"), this, SLOT(pinTab()));
-        menu.addSeparator();
-        menu.addAction(tr("Re&load All Tabs"), m_tabWidget, SLOT(reloadAllTabs()));
-        menu.addAction(tr("&Bookmark This Tab"), this, SLOT(bookmarkTab()));
-        menu.addAction(tr("Bookmark &All Tabs"), m_window, SLOT(bookmarkAllTabs()));
-        menu.addSeparator();
-        menu.addAction(m_window->action(QSL("Other/RestoreClosedTab")));
-        menu.addSeparator();
-        menu.addAction(tr("Close Ot&her Tabs"), this, SLOT(closeAllButCurrent()));
-        menu.addAction(QIcon::fromTheme("window-close"), tr("Cl&ose"), this, SLOT(closeTab()));
-        menu.addSeparator();
-    }
-    else {
-        menu.addAction(tr("Reloa&d All Tabs"), m_tabWidget, SLOT(reloadAllTabs()));
-        menu.addAction(tr("Bookmark &All Tabs"), m_window, SLOT(bookmarkAllTabs()));
-        menu.addSeparator();
-        menu.addAction(m_window->action(QSL("Other/RestoreClosedTab")));
-    }
-
-    m_window->action(QSL("Other/RestoreClosedTab"))->setEnabled(m_tabWidget->canRestoreTab());
+    TabContextMenu menu(index, Qt::Horizontal, m_window, m_tabWidget);
 
     // Prevent choosing first option with double rightclick
     const QPoint pos = event->globalPos();
@@ -399,11 +343,6 @@ void TabBar::updatePinnedTabCloseButton(int index)
     }
 }
 
-void TabBar::closeCurrentTab()
-{
-    m_tabWidget->requestCloseTab(currentIndex());
-}
-
 void TabBar::closeTabFromButton()
 {
     QWidget* button = qobject_cast<QWidget*>(sender());
@@ -433,35 +372,12 @@ void TabBar::currentTabChanged(int index)
         showCloseButton(index);
         hideCloseButton(m_tabWidget->lastTabIndex());
 
-        QTimer::singleShot(100, this, [=]() {
-            ensureVisible(index);
-        });
+        QTimer::singleShot(100, this, [this]() { ensureVisible(); });
     }
 
     m_tabWidget->currentTabChanged(index);
 }
 
-void TabBar::bookmarkTab()
-{
-    TabbedWebView* view = m_window->weView(m_clickedTab);
-    if (!view) {
-        return;
-    }
-
-    WebTab* tab = view->webTab();
-
-    m_window->addBookmark(tab->url(), tab->title());
-}
-
-void TabBar::pinTab()
-{
-    WebTab* webTab = qobject_cast<WebTab*>(m_tabWidget->widget(m_clickedTab));
-
-    if (webTab) {
-        webTab->togglePinned();
-    }
-}
-
 void TabBar::overrideTabTextColor(int index, QColor color)
 {
     if (!m_originalTabTextColor.isValid()) {
@@ -571,7 +487,7 @@ void TabBar::mouseReleaseEvent(QMouseEvent* event)
     }
 
     if (m_tabWidget->buttonAddTab()->isHidden() && !isMainBarOverflowed()) {
-        QTimer::singleShot(500, m_tabWidget->buttonAddTab(), SLOT(show()));
+        QTimer::singleShot(ComboTabBar::slideAnimationDuration(), m_tabWidget->buttonAddTab(), &AddTabButton::show);
     }
 
     if (!rect().contains(event->pos())) {
