@@ -48,6 +48,8 @@
 #include <QWebEngineContextMenuData>
 #include <QStackedLayout>
 #include <QScrollBar>
+#include <QPrintPreviewDialog>
+#include <QPrinter>
 
 bool WebView::s_forceContextMenuOnMouseRelease = false;
 
@@ -143,6 +145,11 @@ void WebView::setPage(WebPage *page)
         return;
     }
 
+    if (m_page) {
+        m_page->setView(nullptr);
+        m_page->deleteLater();
+    }
+
     m_page = page;
     m_page->setParent(this);
     QWebEngineView::setPage(m_page);
@@ -256,15 +263,6 @@ QRect WebView::scrollBarGeometry(Qt::Orientation orientation) const
     return s && s->isVisible() ? s->geometry() : QRect();
 }
 
-void WebView::restoreHistory(const QByteArray &data)
-{
-    QDataStream stream(data);
-    stream >> *history();
-
-    // Workaround clearing QWebChannel after restoring history
-    page()->setupWebChannel();
-}
-
 QWidget *WebView::inputWidget() const
 {
     return m_rwhvqt ? m_rwhvqt : const_cast<WebView*>(this);
@@ -396,6 +394,25 @@ void WebView::forward()
     }
 }
 
+void WebView::printPage()
+{
+    Q_ASSERT(m_page);
+
+    QPrintPreviewDialog* dialog = new QPrintPreviewDialog(this);
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    dialog->resize(800, 750);
+    dialog->printer()->setCreator(tr("QupZilla %1 (%2)").arg(Qz::VERSION, Qz::WWWADDRESS));
+    dialog->printer()->setDocName(QzTools::getFileNameFromUrl(m_page->url()));
+
+    connect(dialog, &QPrintPreviewDialog::paintRequested, this, [=](QPrinter *printer) {
+        QApplication::setOverrideCursor(Qt::WaitCursor);
+        m_page->execPrintPage(printer, 10 * 1000);
+        QApplication::restoreOverrideCursor();
+    });
+
+    dialog->open();
+}
+
 void WebView::slotLoadStarted()
 {
     m_progress = 0;
@@ -403,7 +420,14 @@ void WebView::slotLoadStarted()
 
 void WebView::slotLoadProgress(int progress)
 {
-    m_progress = progress;
+    if (m_progress < 100) {
+        m_progress = progress;
+    }
+
+    // QtWebEngine sometimes forgets applied zoom factor
+    if (!qFuzzyCompare(zoomFactor(), zoomLevels().at(m_currentZoomLevel) / 100.0)) {
+        applyZoom();
+    }
 }
 
 void WebView::slotLoadFinished(bool ok)
@@ -516,14 +540,7 @@ void WebView::showSource()
         return;
     }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 8, 0)
     triggerPageAction(QWebEnginePage::ViewSource);
-#else
-    QUrl u;
-    u.setScheme(QSL("view-source"));
-    u.setPath(url().toString());
-    openUrlInNewTab(u, Qz::NT_SelectedTab);
-#endif
 }
 
 void WebView::showSiteInfo()
@@ -645,7 +662,6 @@ void WebView::createContextMenu(QMenu *menu, WebHitTestResult &hitTest)
     const QWebEngineContextMenuData &contextMenuData = page()->contextMenuData();
     hitTest.updateWithContextMenuData(contextMenuData);
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 8, 0)
     if (!contextMenuData.misspelledWord().isEmpty()) {
         QFont boldFont = menu->font();
         boldFont.setBold(true);
@@ -666,7 +682,6 @@ void WebView::createContextMenu(QMenu *menu, WebHitTestResult &hitTest)
         menu->addSeparator();
         spellCheckActionCount = menu->actions().count();
     }
-#endif
 
     if (!hitTest.linkUrl().isEmpty() && hitTest.linkUrl().scheme() != QL1S("javascript")) {
         createLinkContextMenu(menu, hitTest);
@@ -1239,10 +1254,14 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
 void WebView::loadRequest(const LoadRequest &req)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+    QWebEngineView::load(req.webRequest());
+#else
     if (req.operation() == LoadRequest::GetOperation)
         load(req.url());
     else
         page()->runJavaScript(Scripts::sendPostData(req.url(), req.data()), WebPage::SafeJsWorld);
+#endif
 }
 
 bool WebView::eventFilter(QObject *obj, QEvent *event)
@@ -1307,6 +1326,12 @@ bool WebView::eventFilter(QObject *obj, QEvent *event)
         case QEvent::Wheel:
             return true;
 
+        case QEvent::Hide:
+            if (isFullScreen()) {
+                triggerPageAction(QWebEnginePage::ExitFullScreen);
+            }
+            break;
+
         default:
             break;
         }
