@@ -73,7 +73,6 @@ public:
     void detach();
     void attach(BrowserWindow* window);
 
-    void setHistoryData(const QByteArray &data);
     QByteArray historyData() const;
 
     void stop();
@@ -103,6 +102,8 @@ public:
     void p_restoreTab(const SavedTab &tab);
     void p_restoreTab(const QUrl &url, const QByteArray &history, int zoomLevel);
 
+    void tabActivated();
+
 private slots:
     void showNotification(QWidget* notif);
     void loadStarted();
@@ -112,7 +113,6 @@ private slots:
     void slotRestore();
 
 private:
-    void showEvent(QShowEvent* event);
     void resizeEvent(QResizeEvent *event) override;
 
     BrowserWindow* m_window;
@@ -128,8 +128,6 @@ private:
 
     SavedTab m_savedTab;
     bool m_isPinned;
-
-    static bool s_pinningTab;
 };
 
 #endif // WEBTAB_H
